package net.sf.itcb.addons.monitoring.track.portlet.interceptor;

import org.slf4j.MDC;
import org.springframework.beans.factory.annotation.Required;

import net.sf.itcb.addons.monitoring.track.MonitoringTransactionManager;
import net.sf.itcb.addons.monitoring.track.bean.ItcbTrackBean;
import net.sf.itcb.addons.monitoring.track.bean.LayerType;
import net.sf.itcb.common.business.core.ItcbConstants;
import net.sf.itcb.common.portlet.vaadin.component.ItcbComponent;
import net.sf.itcb.common.portlet.vaadin.interceptor.page.ChangePageInterceptor;
import net.sf.itcb.common.portlet.vaadin.page.PageMappingProcessor.ReloadOrder;


/**
 * This interceptor is used to track each page change in database
 * It is displayed in database as : 
 * <ul>
 *   <li>Method : <code>displayPage(pageRef, reloadOrder, executedAction)</code></li>
 *   <li>Class : <code>net.sf.itcb.addons.monitoring.track.portlet.interceptor.TrackerChangePageInterceptor</code></li>
 * </ul>
 * @author Pierre Le Roux
 */
public class TrackerChangePageInterceptor implements ChangePageInterceptor {

	private static final String CLASS_KEY = "net.sf.itcb.addons.monitoring.track.portlet.interceptor.TrackerChangePageInterceptor";
	
	MonitoringTransactionManager monitoringTransactionManager;
	
	@Required
	public void setMonitoringTransactionManager(MonitoringTransactionManager monitoringTransactionManager) {
		this.monitoringTransactionManager=monitoringTransactionManager;
	}
	
	@Override
	public Object handleBeforeChangePage(String pageRef,
			ItcbComponent itcbComponent, ReloadOrder reloadOrder) {
		 long startTime = System.currentTimeMillis();
		 ItcbTrackBean itcbTrackBean = new ItcbTrackBean();
		 itcbTrackBean.setClassName(CLASS_KEY);
		 itcbTrackBean.setLayerType(LayerType.VAADIN);
		 itcbTrackBean.setStartTime(startTime);
		 monitoringTransactionManager.setCurrentTrackBean(itcbTrackBean);
		return itcbTrackBean;
	}

	@Override
	public void handleAfterChangePage(String pageRef,
			ItcbComponent itcbComponent, ReloadOrder reloadOrder,
			ExecutedAction executedAction, Object requestContextVar) {
		
		ItcbTrackBean itcbTrackBean = (ItcbTrackBean)requestContextVar;
		itcbTrackBean.setTimeInMillis(System.currentTimeMillis() - itcbTrackBean.getStartTime());
		itcbTrackBean.setMethod("displayPage("+pageRef+","+reloadOrder+"," + executedAction + ")");
		itcbTrackBean.setParameters(new Object[] { executedAction } );
		itcbTrackBean.setUser(MDC.get(ItcbConstants.MDC_USER_KEY));
		monitoringTransactionManager.endCurrentTrackBean();

	}

	@Override
	public void handleChangePageException(String pageRef,
			ItcbComponent itcbComponent, ReloadOrder reloadOrder,
			ExecutedAction executedAction, Object requestContextVar,
			Throwable throwable) {
		ItcbTrackBean itcbTrackBean = (ItcbTrackBean)requestContextVar;
		itcbTrackBean.setException(throwable);
		handleAfterChangePage(pageRef, itcbComponent, reloadOrder, executedAction, requestContextVar);

	}

}
